package de.queuedbit.barometer_plugin;

import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.util.Log;

import androidx.annotation.NonNull;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import static android.content.Context.SENSOR_SERVICE;
/** BarometerPlugin */
public class BarometerPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware  {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;
  private Context context;
  private float[] values;
  private SensorEventListener sensorEventListener;
  private SensorManager sensorManager;
  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "barometer_plugin");
    channel.setMethodCallHandler(this);
    context = flutterPluginBinding.getApplicationContext();
  }

  // This static function is optional and equivalent to onAttachedToEngine. It supports the old
  // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
  // plugin registration via this function while apps migrate to use the new Android APIs
  // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
  //
  // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
  // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
  // depending on the user's project. onAttachedToEngine or registerWith must both be defined
  // in the same class.
  public static void registerWith(Registrar registrar) {
    final MethodChannel channel = new MethodChannel(registrar.messenger(), "barometer_plugin");
    channel.setMethodCallHandler(new BarometerPlugin());
  }


  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    if (call.method.equals("getPlatformVersion")) {
      result.success("Android " + android.os.Build.VERSION.RELEASE);
    }
    else if(call.method.equals("getBarometerData")){
      result.success(convertFloatsToDoubles(values));
    }
    else if(call.method.equals("stopBarometerListener")){
      cancelListener();
    }
    else if(call.method.equals("startBarometerListener")){
      int updateSpeed = call.argument("updateSpeed");
      registerBarometerListener(updateSpeed);
    }
    else {
      result.notImplemented();
    }
  }

  private void cancelListener(){
    if(sensorEventListener != null && sensorManager != null){
      sensorManager.unregisterListener(sensorEventListener);
      values = null;
    }
  }

  public static double[] convertFloatsToDoubles(float[] input)
  {
    if (input == null)
    {
      return null; // Or throw an exception - your choice
    }
    double[] output = new double[input.length];
    for (int i = 0; i < input.length; i++)
    {
      output[i] = input[i];
    }
    return output;
  }

  private void registerBarometerListener(final int updateSpeed){
    sensorManager = (SensorManager) context.getSystemService(SENSOR_SERVICE);
    Sensor pressureSensor = sensorManager.getDefaultSensor(Sensor.TYPE_PRESSURE);
    sensorEventListener = new SensorEventListener() {
      @Override
      public void onSensorChanged(SensorEvent sensorEvent) {
        values = sensorEvent.values;
        //Log.d("", updateSpeed+"Debugging "+String.format("%.3f mbar", values[0]));
      }

      @Override
      public void onAccuracyChanged(Sensor sensor, int i) {

      }
    };

    int sensorSpeed;
    switch(updateSpeed) {
      case 0:
        sensorSpeed = SensorManager.SENSOR_DELAY_FASTEST;
        break;
      case 1:
        sensorSpeed = SensorManager.SENSOR_DELAY_GAME;
        break;
      case 2:
        sensorSpeed = SensorManager.SENSOR_DELAY_UI;
        break;
      case 3:
        sensorSpeed = SensorManager.SENSOR_DELAY_NORMAL;
        break;
      default:
        sensorSpeed = SensorManager.SENSOR_DELAY_FASTEST;
        break;
    }
    sensorManager.registerListener(sensorEventListener, pressureSensor, sensorSpeed);

  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }

  @Override
  public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {

  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {

  }

  @Override
  public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {

  }

  @Override
  public void onDetachedFromActivity() {

  }
}
